#ifndef _BADGE_V2_H_
#define _BADGE_V2_H_

// Charge les librairies des polices de caractères
// Attention ces polices de caractères n'intègrent pas les caractères accentués

#include <Fonts/FreeMono9pt7b.h>
#include <Fonts/FreeMonoBold9pt7b.h>
#include <Fonts/FreeMonoBold12pt7b.h>
#include <Fonts/FreeSerifBold12pt7b.h>
#include <Fonts/FreeSerifBoldItalic24pt7b.h>

// Attribut des polices de caractères aux textes du badge

	const GFXfont *titleFont PROGMEM    = &FreeSerifBold12pt7b;
	const GFXfont *callSignFont PROGMEM = &FreeSerifBoldItalic24pt7b;
	const GFXfont *qthLocFont PROGMEM   = &FreeMonoBold9pt7b;
	const GFXfont *fNameFont PROGMEM    = &FreeMonoBold12pt7b;
	const GFXfont *refNumFont PROGMEM   = &FreeMonoBold9pt7b;
	const GFXfont *acronymFont PROGMEM  = &FreeSerifBold12pt7b;
	const GFXfont *freetextFont PROGMEM = &FreeMono9pt7b;
	
	const int     logoSize_X            = X_SIZE;	  		  // Taille en X du logo du REF
	const int     logoSize_Y            = Y_SIZE;	  		  // Taille en Y du logo du REF

	const char    cFileName[]   		    = "/badge.json";  // Fichier JSON à charger

#define MAX_ROOT_JSON_SIZE  1024                        // Taille max du buffer de désérialisation du fichier JSON
#define	MAXT_TITLE			    17
#define	MAXT_CALLSIGN		    7
#define	MAXT_FIRST_NAME		  22
#define	MAXT_QTH_LOC		    7
#define MAXT_REF_NUMBER		  6
#define	MAXT_ACRONYM		    4
#define	MAXT_FREE_TEXT		  22

typedef struct __BadgeConfig__
{
    char        sTitle[MAXT_TITLE + 1];               // Titre du badge
    int         cTitle_X      ;                       // Position du texte du titre en X
    int         cTitle_Y      ;                       // Position du texte du titre en Y
    bool        bTLEnable     ;                       // Active ou non l'affichage du titre du badge

    char        sCallSign[MAXT_CALLSIGN + 1];         // Indicatif - Compatible D-STAR sans le code commande
    int         cSign_X       ;                       // Position du texte de l'indicatif en X
    int         cSign_Y       ;                       // Position du texte de l'indicatif en Y
    bool        bCSEnable     ;                       // Active ou non l'affichage de l'indicatif

    char        sName[MAXT_FIRST_NAME + 1] ;          // Prénom de l'OM
    int         cName_X       ;                       // Position du texte du prénom en X
    int         cName_Y       ;                       // Position du texte du prénom en Y
    bool        bCNEnable     ;                       // Active ou non l'affichage du prénom

    char        qthLoc[MAXT_QTH_LOC + 1] ;            // QTH locator
    int         qthLoc_X      ;                       // Position du texte du QTH LOC en X
    int         qthLoc_Y      ;                       // Position du texte du QTH LOC en Y
    bool        bQLEnable     ;                       // Active ou non l'affichage du QTH locator

    char        refNumber[MAXT_REF_NUMBER + 1];       // N° adhérent au REF
    int         refNum_X      ;                       // Position du texte du N° du REF en X
    int         refNum_Y      ;                       // Position du texte du N° du REF en Y
    bool        bRefRegister  ;                       // Active ou non l'affichage des infos du REF

    int         logoPos_X     ;                       // Position du logo en X
    int         logoSize_X    ;                       // Taille du logo en X
    int         logoPos_Y     ;                       // Position du logo en Y
    int         logoSize_Y    ;                       // Taille du logo en Y
    bool        bLGReverse    ;                       // Affichage en mode normal ou inverse du logo
    bool        bLGEnable     ;                       // Active ou non l'affichage du logo

    char        acronym[MAXT_ACRONYM + 1];            // Texte du sigle inférieur ou egal à 4 lettres
    int         acronym_X     ;                       // Position du texte du sigle en X
    int         acronym_Y     ;                       // Position du texte du sigle en Y
    int         acronymHeight ;                       // Hauteur de chaque lettre
    bool        bAMEnable     ;                       // Active ou non l'affichage du sigle

    char        freetext[MAXT_FREE_TEXT + 1];         // Texte libre de 22 caractères MAX
    int         freetext_X    ;                       // Posision du texte libre en X
    int         freetext_Y    ;                       // Position du texte libre en Y
    bool        bFTEnable     ;                       // Active ou non le texte libre
} CONFIG, *LPCONFIG;

        bool        bSdOk         = false;            // Par défaut pas de carte SD détectée
        CONFIG      Config;                           // Configuration d'affichage en provenance de la carte SD
        File        configFile;                       // Pointeur sur le fichier de configuration
        SPIClass    SDSPI(VSPI);                      // Objet pour la gestion de la carte SD

/******************************************************************/
//  Code de lecture du fichier JSON - F5FDR - 01/02/2023

// Charge la configuration par défaut avant la lecture de la carte SD
void loadDefaultValue(LPCONFIG lpConfig)
{
  if (lpConfig != NULL) {
  
    memset((uint8_t *)lpConfig, 0, sizeof(CONFIG));

    // Titre du badge
    strcpy(lpConfig->sTitle, titleString);
    lpConfig->cTitle_X = cTitle_X;
    lpConfig->cTitle_Y = cTitle_Y;
    lpConfig->bTLEnable = bTLEnable;
    
    // Indicatif
    strcpy(lpConfig->sCallSign, callSign); 
    lpConfig->cSign_X = cSign_X;
    lpConfig->cSign_Y = cSign_Y;
    lpConfig->bCSEnable = bCSEnable;
  
    // Prénom
    strcpy(lpConfig->sName, firstName);
    lpConfig->cName_X = fName_X;
    lpConfig->cName_Y = fName_Y;
    lpConfig->bCNEnable = bCSEnable;
  
    // N° d'adhérent au REF
    strcpy(lpConfig->refNumber, refNumber);
    lpConfig->refNum_X = refNum_X;
    lpConfig->refNum_Y = refNum_Y;
    lpConfig->bRefRegister = bRefRegister;

    // Logo du REF
    lpConfig->logoPos_X = logoPos_X;
    lpConfig->logoSize_X = logoSize_X;
    lpConfig->logoPos_Y = logoPos_Y;
    lpConfig->logoSize_Y = logoSize_Y;
    lpConfig->bLGReverse = bLGReverse;
    lpConfig->bLGEnable = bLGEnable;
    
    // QTH Locator
    strcpy(lpConfig->qthLoc, qthLoc);
    lpConfig->qthLoc_X = qthLoc_X;
    lpConfig->qthLoc_Y = qthLoc_Y;
    lpConfig->bQLEnable = bQLEnable;

    // Sigle 
    strcpy(lpConfig->acronym, acronym);
    lpConfig->acronym_X = acronym_X;
    lpConfig->acronym_Y = acronym_Y;
    lpConfig->acronymHeight = acronymHeight;
    lpConfig->bAMEnable = bAMEnable;

    // Texte libre
    strcpy(lpConfig->freetext, freetext);
    lpConfig->freetext_X = freetext_X;
    lpConfig->freetext_Y = freetext_Y;
    lpConfig->bFTEnable = bFTEnable;
  }
}

// Carte SD: Charge le texte libre
bool getFreeTextInfos(LPCONFIG lpConfig, JsonObject dataFreeText)
{
  if ((lpConfig != NULL) && !dataFreeText.isNull()) {
    strlcpy(lpConfig->freetext, dataFreeText["freetext"] | freetext, sizeof(lpConfig->freetext));
    dataFreeText["x_pos"].isNull() ? lpConfig->freetext_X = freetext_X : lpConfig->freetext_X = dataFreeText["x_pos"].as<int>();
    dataFreeText["y_pos"].isNull() ? lpConfig->freetext_Y = freetext_Y : lpConfig->freetext_Y = dataFreeText["y_pos"].as<int>();
    dataFreeText["enable"].isNull() ? lpConfig->bFTEnable = bFTEnable : lpConfig->bFTEnable = dataFreeText["enable"].as<bool>();
    return(true);
  }
#if (__SERIAL_DEBUG__ == true)
  Serial.println("JSON:Failed to loading free text");
#endif 
  return(false);
}

// Carte SD: Charge le sigle à afficher
bool getAcronymInfos(LPCONFIG lpConfig, JsonObject dataAcronym)
{
  if ((lpConfig != NULL) && !dataAcronym.isNull()) {
    strlcpy(lpConfig->acronym, dataAcronym["text"] | refNumber, sizeof(lpConfig->acronym));
    dataAcronym["x_pos"].isNull() ? lpConfig->acronym_X = acronym_X : lpConfig->acronym_X = dataAcronym["x_pos"].as<int>();
    dataAcronym["y_pos"].isNull() ? lpConfig->acronym_Y = acronym_Y : lpConfig->acronym_Y = dataAcronym["y_pos"].as<int>();
    dataAcronym["height"].isNull() ? lpConfig->acronymHeight = acronymHeight : lpConfig->acronymHeight = dataAcronym["height"].as<int>();
    dataAcronym["enable"].isNull() ? lpConfig->bAMEnable = bAMEnable : lpConfig->bAMEnable = dataAcronym["enable"].as<bool>();
    return(true);
  }
#if (__SERIAL_DEBUG__ == true)
  Serial.println("JSON:Failed to read acronym infos");
#endif
  return(false);
}

// Carte SD: Charge le QTH locator
bool getQthLocInfos(LPCONFIG lpConfig, JsonObject dataQth)
{
  if ((lpConfig != NULL) && !dataQth.isNull()) {
    strlcpy(lpConfig->qthLoc, dataQth["qth"] | refNumber, sizeof(lpConfig->qthLoc));
    dataQth["x_pos"].isNull() ? lpConfig->qthLoc_X = qthLoc_X : lpConfig->qthLoc_X = dataQth["x_pos"].as<int>();
    dataQth["y_pos"].isNull() ? lpConfig->qthLoc_Y = qthLoc_Y : lpConfig->qthLoc_Y = dataQth["y_pos"].as<int>();
    dataQth["enable"].isNull() ? lpConfig->bQLEnable = bQLEnable : lpConfig->bQLEnable = dataQth["enable"].as<bool>();
    return(true);
  }
#if (__SERIAL_DEBUG__ == true)
  Serial.println("JSON:Failed to read QTH locator infos");
#endif
  return(false);
}

// Carte SD: Charge les infos d'adhésion au REF
bool getRefInfos(LPCONFIG lpConfig, JsonObject dataRef)
{
  if ((lpConfig != NULL) && !dataRef.isNull()) {
    strlcpy(lpConfig->refNumber, dataRef["refnum"] | refNumber, sizeof(lpConfig->refNumber));
    dataRef["x_pos"].isNull() ? lpConfig->refNum_X = refNum_X : lpConfig->refNum_X = dataRef["x_pos"].as<int>();
    dataRef["y_pos"].isNull() ? lpConfig->refNum_Y = refNum_Y : lpConfig->refNum_Y = dataRef["y_pos"].as<int>();
    dataRef["enable"].isNull() ? lpConfig->bRefRegister = bRefRegister : lpConfig->bRefRegister = dataRef["enable"].as<bool>();
    return(true);
  }
#if (__SERIAL_DEBUG__ == true)
  Serial.println("JSON:Failed to read REF infos");
#endif
  return(false);
}

// Carte SD: Charge les infos du logo
bool getLogo(LPCONFIG lpConfig, JsonObject dataLogo)
{
  if ((lpConfig != NULL) && !dataLogo.isNull()) {
    dataLogo["x_pos"].isNull() ? lpConfig->logoPos_X = logoPos_X : lpConfig->logoPos_X = dataLogo["x_pos"].as<int>();
    dataLogo["x_size"].isNull() ? lpConfig->logoSize_X = logoSize_X : lpConfig->logoSize_X = dataLogo["x_size"].as<int>();
    dataLogo["y_pos"].isNull() ? lpConfig->logoPos_Y = logoPos_Y : lpConfig->logoPos_Y = dataLogo["y_pos"].as<int>();
    dataLogo["y_size"].isNull() ? lpConfig->logoSize_Y = logoSize_Y : lpConfig->logoSize_Y = dataLogo["y_size"].as<int>();
    dataLogo["reverse"].isNull() ? lpConfig->bLGReverse = bLGReverse : lpConfig->bLGReverse = dataLogo["reverse"].as<bool>();
    dataLogo["enable"].isNull() ? lpConfig->bLGEnable = bLGEnable : lpConfig->bLGEnable = dataLogo["enable"].as<bool>();
    return(true);
  }
#if (__SERIAL_DEBUG__ == true)
  Serial.println("JSON:Failed to read logo infos");
#endif
  return(false);
}

// Carte SD: Charge le titre du badge
bool getTitle(LPCONFIG lpConfig, JsonObject dataTitle)
{
  if ((lpConfig != NULL) && !dataTitle.isNull()) {
    strlcpy(lpConfig->sTitle, dataTitle["title"] | titleString, sizeof(lpConfig->sTitle));
    dataTitle["x_pos"].isNull() ? lpConfig->cTitle_X = cTitle_X : lpConfig->cTitle_X = dataTitle["x_pos"].as<int>();
    dataTitle["y_pos"].isNull() ? lpConfig->cTitle_Y = cTitle_Y : lpConfig->cTitle_Y = dataTitle["y_pos"].as<int>();
    dataTitle["enable"].isNull() ? lpConfig->bTLEnable = bTLEnable : lpConfig->bTLEnable = dataTitle["enable"].as<bool>();
    return(true);
  }
#if (__SERIAL_DEBUG__ == true)
  Serial.println("JSON:Failed to read title infos");
#endif
  return(false);
}

// Carte SD: Charge le prénom de l'OM
bool getFirstName(LPCONFIG lpConfig, JsonObject dataName)
{
  if ((lpConfig != NULL) && !dataName.isNull()) {
    strlcpy(lpConfig->sName, dataName["myname"] | firstName, sizeof(lpConfig->sName));
    dataName["x_pos"].isNull() ? lpConfig->cName_X = fName_X : lpConfig->cName_X = dataName["x_pos"].as<int>();
    dataName["y_pos"].isNull() ? lpConfig->cName_Y = fName_Y : lpConfig->cName_Y = dataName["y_pos"].as<int>();
    dataName["enable"].isNull() ? lpConfig->bCNEnable = bCSEnable : lpConfig->bCNEnable = dataName["enable"].as<bool>();
    return(true);
  }
#if (__SERIAL_DEBUG__ == true)
  Serial.println("JSON:Failed to read name infos");
#endif
  return(false);
}

// Carte SD: Charge l'indicatif
bool getCallSign(LPCONFIG lpConfig, JsonObject dataCfg)
{
  if ((lpConfig != NULL) && !dataCfg.isNull()) {
    strlcpy(lpConfig->sCallSign, dataCfg["mycall"] | callSign, sizeof(lpConfig->sCallSign));
    dataCfg["x_pos"].isNull() ? lpConfig->cSign_X = cSign_X : lpConfig->cSign_X = dataCfg["x_pos"].as<int>();
    dataCfg["y_pos"].isNull() ? lpConfig->cSign_Y = cSign_Y : lpConfig->cSign_Y = dataCfg["y_pos"].as<int>();
    dataCfg["enable"].isNull() ? lpConfig->bCSEnable = bCSEnable : lpConfig->bCSEnable = dataCfg["enable"].as<bool>();
    return(true);
  }
#if (__SERIAL_DEBUG__ == true)
    Serial.println("JSON:Failed to read callsign infos");
#endif
  return(false);
}

// Active la gestion de la carte SD
bool setupSDCard(void)
{
  SDSPI.begin(SDCARD_SCLK, SDCARD_MISO, SDCARD_MOSI);
  return(SD.begin(SDCARD_CS, SDSPI));
}

// Charge le fichier JSON utilisateur
bool loadUserJsonFile(File hFile, LPCONFIG lpConfig)
{
  if ((hFile != NULL) && (lpConfig != NULL)) {
#if (__SERIAL_DEBUG__ == true)
    Serial.println("JSON:Read config file");
#endif
    StaticJsonDocument<MAX_ROOT_JSON_SIZE> root;                        // Création du buffer de lecture du JSON
    DeserializationError error = deserializeJson(root, hFile);          // Lecture (désérialisation) du fichier JSON

    if (error) {
#if (__SERIAL_DEBUG__ == true)
      Serial.println("JSON:Failed to read file, using default configuration");
#endif
      return(false);
    }
    else {
      getTitle(lpConfig,        root["title"].as<JsonObject>());
      getCallSign(lpConfig,     root["callsign"].as<JsonObject>());
      getFirstName(lpConfig,    root["name"].as<JsonObject>());
      getRefInfos(lpConfig,     root["ref"].as<JsonObject>());
      getLogo(lpConfig,         root["logo"].as<JsonObject>());
      getQthLocInfos(lpConfig,  root["qthloc"].as<JsonObject>());
      getAcronymInfos(lpConfig, root["acronym"].as<JsonObject>());
      getFreeTextInfos(lpConfig,root["freetext"].as<JsonObject>());
      return(true);
    }
  }
#if (__SERIAL_DEBUG__ == true)
  Serial.println("JSON:Input parameters error");
#endif
  return(false);
}

// Recherche le fichier JSON sur la carte SD et le charge si trouvé
bool checkUserJsonFile(File hFile, bool *bError)
{
  if (SD.exists(cFileName)) {
#if (__SERIAL_DEBUG__ == true)
    Serial.println("SD:Config file found");
#endif
    configFile = SD.open(cFileName, FILE_READ);
    if (configFile) {
      if (!loadUserJsonFile(configFile, &Config)) {
#if (__SERIAL_DEBUG__ == true)
        Serial.println("SD:Reading JSON file error");
#endif
        *bError = false;
      }
      configFile.close();                           // Fermeture du fichier.
    }
    else {
#if (__SERIAL_DEBUG__ == true)
      Serial.println("SD:File open error.");
#endif
      *bError = false;
    }
  }
  else {
#if (__SERIAL_DEBUG__ == true)
    Serial.println("SD:File not found.");
#endif
    *bError = false;
  }
}

/******************************************************************/
//  Code d'affichage - F5FDR - 30/01/2023

// Le logo est codé en RVB 8 bit par couleur. Cette routine
// extrait un octet sur trois pour l'affichage en monochrome.
// La valeur lue sera soit 0x00 soit 0xFF pour un affichage
// monochrome sans niveaux de gris (0x00 == noir / 0xFF == blanc)
// Si "bReverse" == true, le logo sera sur fond noir (mode inversé)

void displayLogo(LPCONFIG lpConfig)
{
  if (lpConfig != NULL) {
    int                   i,n;                          // Variable de calcul
    int                   x = lpConfig->logoPos_X;      // Position X du logo sur l'afficheur
    int                   y = lpConfig->logoPos_Y;      // Position Y du logo sur l'afficheur
    const unsigned char   *p = ref;                     // Init pointeur sur le tableau des bits RVB du logo (3 x octets / pixels)
  
    for (i = 0; i < (lpConfig->logoSize_X * lpConfig->logoSize_Y); i++) { // Lecture de la matrice (taille 50 points par 71 lignes graphiques) 
      for (n = 0; n < NB_COLORS; n++) {                 // Lecture des trois valeurs RVB
        if (n == 0) {                                   // Traitement pour la première valeur / les deux autres sont ignorés
          if (*p != (lpConfig->bLGReverse ? 0x00 : 0xFF)) {  // Gestion du mode normal / reverse
            display.writePixel(x, y, GxEPD_BLACK);      // Affichage du pixel noir
          }
        }
        p++;                                            // Avance d'une position dans le tableau des valeurs RVB
      }
      x++;                                              // Avance d'un pixel sur l'afficheur
      if (x >= (lpConfig->logoPos_X + lpConfig->logoSize_X)) { // Fin de ligne atteinte de (n) pixels ? (Oui) 
        x = lpConfig->logoPos_X;                        // Retour en début de ligne...
        y++;                                            // et passe à la ligne suivante
      }
    }
  }
}

void displayCallSign(LPCONFIG lpConfig)
{
  if (lpConfig != NULL) {
#if (__SERIAL_DEBUG__ == true)
    Serial.println("Display Callsign");
#endif

    // Sélection du type d'affichage (encre noire sur fond blanc)
    display.setTextColor(GxEPD_BLACK);                            // Type d'encre pour l'écriture (Noir)
    display.fillScreen(GxEPD_WHITE);                              // Type de fond pour l'écriture (Blanc)

    // Affichage du titre du badge
    if (lpConfig->bTLEnable) {                                    // Affichage de l'indicatif activé ? (Oui)
      display.setFont(titleFont);                                 // Choix de la police de caractère
      display.setCursor(lpConfig->cTitle_X, lpConfig->cTitle_Y);  // Position X et Y de texte du titre du badge
      display.print(lpConfig->sTitle);                            // Texte du titre du badge
    }
    
    // Affichage de l'indicatif
    if (lpConfig->bCSEnable) {                                    // Affichage de l'indicatif activé ? (Oui)
      display.setFont(callSignFont);                              // Choix de la police de caractère
      display.setCursor(lpConfig->cSign_X, lpConfig->cSign_Y);    // Position X et Y du texte de l'indicatif
      display.print(lpConfig->sCallSign);                         // Texte de l'indicatif
    }

    // Affichage du n° du REF
    if (lpConfig->bRefRegister) {
      display.setFont(qthLocFont);                                // Choix de la police de caractère
      display.setCursor(lpConfig->refNum_X, lpConfig->refNum_Y);  // Position X et Y du texte du n° du REF
      display.print(lpConfig->refNumber);                         // Texte du n° du REF
    }

    // Affichage du QTH Locator
    if (lpConfig->bQLEnable) {
      display.setFont(qthLocFont);                                // Choix de la police de caractère
      display.setCursor(lpConfig->qthLoc_X, lpConfig->qthLoc_Y);  // Position X et Y du texte du QTH Locator
      display.print(lpConfig->qthLoc);                            // Texte du QTH Locator
    }
   
    // Affichage du prénom
    if (lpConfig->bCNEnable) {
      display.setFont(fNameFont);                                 // Choix de la police de caractère
      display.setCursor(lpConfig->cName_X, lpConfig->cName_Y);    // Position X et Y du texte du prénom
      display.print(lpConfig->sName);                             // Texte du prénom
    }

    // Affichage du Logo de REF
    if (lpConfig->bLGEnable) {                                    // Affichage du logo activé ? (Oui)
      displayLogo(lpConfig);                                      // Affichage du Logo du REF en mode normal (ou inverse si true)
    }

    // Affichage du sigle (Texte uniquement et verticalement)
    if (lpConfig->bAMEnable) {                                    // Affichage de texte activé ? (Oui)
      uint8_t     i;                                              // Compteur de caractère de la chaine de texte
      int         yPos = lpConfig->acronym_Y;                     // Position verticale de départ
      const char  *p = lpConfig->acronym;                         // Pointeur sur la chaine de texte
  
      display.setFont(acronymFont);                               // Choix de la police de caractère
      display.setCursor(lpConfig->acronym_X, yPos);               // Position X et Y du texte de la première letre du texte
      for (i = 0; i < sizeof(lpConfig->acronym); i++) {           // Boucle d'écriture du texte pour les (n) caractères
        display.print(*p++);                                      // Afiche le caractère du texte
        yPos += lpConfig->acronymHeight;                          // Position suivante sous le caractère précédent (doit prendre en compte la hauteur de la police de caractère et l'espacement entre deux caractère)
        display.setCursor(lpConfig->acronym_X, yPos);             // Position suivante à écrire
      }
    }

    // Affichage du texte libre
    if (lpConfig->bFTEnable) {                                    // Affichage de texte activé ? (Oui)
      display.setFont(qthLocFont);                                // Choix de la police de caractère
      display.setCursor(lpConfig->freetext_X, lpConfig->freetext_Y);  // Position X et Y du texte libre
      display.print(lpConfig->freetext);                          // Texte libre
    }

    // Affichage du Badge
    display.display(true);                                        // Mise à jour de l'afficheur et affiche tous les textes et logo
  }
}

#endif  //_BADGE_V2_H_
